################################################
##                                            ##
##   PROCESS BIOREMEDIATED MODELS             ##
##                                            ##
################################################

#function to write diet matrix and bioconcentrations in tabular form
#for processing with circos tableviewer
#dietMatrix: contaminant uptake from diet
#TL: trophic levels
writeCircosOutput <- function(dietMatrix,TL,outPath){
  adjMatrix<-dietMatrix
  #zero flows to detritus groups
  adjMatrix[,"DETRITUS"]<-0
  adjMatrix[,"DISCARD"]<-0
  #replace group names with IDs 
  rownames(adjMatrix)<-1:39
  colnames(adjMatrix)<-1:39
  #order by trophic level
  adjMatrix<-adjMatrix[order(TL),order(TL)]
  #write file
  write.csv(adjMatrix,paste(outPath,"_adjMatrix.csv",sep=""))
}
#102 networks are provided corresponding to 51 different levels of bioremediation
#efficiency for the 2 scenarios
#####
#res_trophic: trophic network estimation results
#res_contaminant: contaminant network estimation results
#res_TL: trophic levels
processBioremediationNetworks <- function(res_trophic, res_contaminant, res_TL){
  
  #empty matrices to store bioconcentrations of species in the remediated networks.
  bioconcs_det<-matrix(0,nrow=nrow(res_contaminant$flowM_no_ext),ncol=51)
  bioconcs_water<-matrix(0,nrow=nrow(res_contaminant$flowM_no_ext),ncol=51)
  
  #empty matrices to store flow betweennes centralities of species in the remediated networks.
  fbcs_det<-matrix(0,nrow=nrow(res_contaminant$flowM_no_ext),ncol=51)
  fbcs_water<-matrix(0,nrow=nrow(res_contaminant$flowM_no_ext),ncol=51)
  
  #empty vectors to store link densities in the remediated networks.
  lds_det<-rep(0,times=51)
  lds_water<-rep(0,times=51)
  
  #color palette for bioconcentration plots
  colorFun <- colorRampPalette(c("#80D7FF","#DD1D20"))
  
  #export network with no bioremediation (scenario 0) to circos tableviewer format
  #Fig 4a
  writeCircosOutput(res_contaminant$dietcomposition,res_TL$TL,"outputCircos/scenario0")
  
  #process files related to bioremediation of detritus (scenario 1)
  for(i in 1:51){
    fwFluxes_1<-parseFlowMatrixFromReactionList(path=paste("fluxes_biorem/FW_fluxes_detritus_",i,".csv",sep=""),
                                                numCompartments=nrow(res_contaminant$flowM),
                                                names=rownames(res_contaminant$flowM),
                                                biomassFlwMatrix=res_trophic$flowM,
                                                res_contaminant=res_contaminant,
                                                outdir="outputs_biorem_FW/",
                                                res_TL=res_TL)
    #fetch and store FBC, bioconcentrations and link density
    fbcs_det[,i]<-fwFluxes_1$fbc
    bioconcs_det[,i]<-fwFluxes_1$bioconcs$Conc
    lds_det[i]<-fwFluxes_1$ld  
  }
  
  rownames(fbcs_det)<-1:nrow(fbcs_det)
  rownames(bioconcs_det)<-1:nrow(bioconcs_det)
  colnames(fbcs_det)<-paste((1:ncol(fbcs_det)-1)*2,"%",sep="")
  colnames(fbcs_det)[1:ncol(fbcs_det)%%2==0]<-""
  colnames(bioconcs_det)<-paste((1:ncol(bioconcs_det)-1)*2,"%",sep="")
  colnames(bioconcs_det)[1:ncol(bioconcs_det)%%2==0]<-""
  
  #export network at max bioremediation (scenario 1) to circos tableviewer format
  #Fig 4b
  writeCircosOutput(fwFluxes_1$dietMatrix,res_TL$TL,"outputCircos/scenario1")
  
  #process files related to bioremediation of water (scenario 2)
  for(i in 1:51){
    fwFluxes_2<-parseFlowMatrixFromReactionList(path=paste("fluxes_biorem/FW_fluxes_water_",i,".csv",sep=""),
                                                numCompartments=nrow(res_contaminant$flowM),
                                                names=rownames(res_contaminant$flowM),
                                                biomassFlwMatrix=res_trophic$flowM,
                                                res_contaminant=res_contaminant,
                                                outdir="outputs_biorem_FW/",
                                                res_TL=res_TL)
    #fetch and store FBC, bioconcentrations and link density
    fbcs_water[,i]<-fwFluxes_2$fbc
    bioconcs_water[,i]<-fwFluxes_2$bioconcs$Conc
    lds_water[i]<-fwFluxes_2$ld
  }
  rownames(fbcs_water)<-1:nrow(fbcs_water)
  rownames(bioconcs_water)<-1:nrow(bioconcs_water)
  colnames(fbcs_water)<-paste((1:ncol(fbcs_water)-1)*2,"%",sep="")
  colnames(fbcs_water)[1:ncol(fbcs_water)%%2==0]<-""
  colnames(bioconcs_water)<-paste((1:ncol(bioconcs_water)-1)*2,"%",sep="")
  colnames(bioconcs_water)[1:ncol(bioconcs_water)%%2==0]<-""
  
  #export network at max bioremediation (scenario 2) to circos tableviewer format
  #Fig 4c
  writeCircosOutput(fwFluxes_2$dietMatrix,res_TL$TL,"outputCircos/scenario2")
  return(list(bioconcs_det=bioconcs_det,fbcs_det=fbcs_det,bioconcs_water=bioconcs_water,fbcs_water=fbcs_water))
}

#display plots of networks after bioremediation (fig 5)
#remediatedModel: results of bioconcentrations and network analysis in food webs
#after bioremedation (both scenarios)
plotNetworksBioremediation <- function(remediatedModel){
  bioconcs_det<-remediatedModel$bioconcs_det
  fbcs_det<-remediatedModel$fbcs_det
  bioconcs_water<-remediatedModel$bioconcs_water
  fbcs_water<-remediatedModel$fbcs_water
  #FBC are plotted on a logarithmic scale
  fbcs_det_log<-log(fbcs_det+1)
  fbcs_water_log<-log(fbcs_water+1)
  
  #compute max FBC and bioconcentration
  maxFBC<-max(fbcs_det_log,fbcs_water_log)
  maxBioc<-max(bioconcs_det,bioconcs_water)
  #set colorkey labels
  key_points=unique(c(seq(0, maxFBC, length=10)))
  key_labels=as.character(round(exp(key_points)-1,digits=2))
  
  #max and min bioconcentrations and link densities, to have same scale
  bioconc_min<-min(colSums(bioconcs_water),colSums(bioconcs_det))
  bioconc_max<-max(colSums(bioconcs_water),colSums(bioconcs_det))
  ld_min<-min(lds_water,lds_det)
  ld_max<-max(lds_water,lds_det)
  
  #plot sum of bioconcentrations in scenario 1
  plot(colSums(bioconcs_det),type="l", lwd=1, xlab="", xaxt="n", 
       yaxt="n",ylab="",ylim=c(bioconc_min,bioconc_max))
  polygon(cbind(c(1,1:51,51),c(0,colSums(bioconcs_det),0)),lwd=1, col="#DD1D20")
  
  #bioconcentrations in scenario 1 levelplot, Fig. 5a
  plota<-levelplot(bioconcs_det, at=unique(c(seq(0, maxBioc, length=100))),
                   main="Fig 5a",
                   scales=list(fontfamily="Arial",x=list(rot=90)),
                   col.regions=colorFun(100), aspect=1,  contour=FALSE, pretty=TRUE,
                   border='black',
                   xlab="PCBs (ng/g WW)", 
                   ylab="Remediated Flow")
  
  #FBCs in scenario 1 levelplot, Fig. 5b
  plotb<-levelplot(fbcs_det_log, scales=list(fontfamily="Arial",x=list(rot=90)),
                   main="Fig 5b",
                   colorkey=list(at=key_points, labels=key_labels),
                   at=unique(c(seq(0, maxFBC, length=100))),
                   col.regions=topo.colors(130)[30:130], aspect=1,  contour=FALSE, pretty=TRUE,
                   xlab=expression("FBC" (mg/km^{2}/y)), border='black',
                   ylab="Remediated Flow")
  
  #plot link densities in scenario 1
  plot(lds_det,type="l", lwd=1, xlab="", xaxt="n", 
       yaxt="n",ylab="",ylim=c(ld_min,ld_max))
  polygon(cbind(c(1,1:51,51),c(0,lds_det,0)),lwd=1, col="#104EFF")
  
  #plot sum of bioconcentrations in scenario 2
  plot(colSums(bioconcs_water),type="l", lwd=1, xlab="", xaxt="n", 
       yaxt="n",ylab="",ylim=c(bioconc_min,bioconc_max))
  polygon(cbind(c(1,1:51,51),c(0,colSums(bioconcs_water),0)),lwd=1, col="#DD1D20")
  
  #bioconcentrations in scenario 2 levelplot, Fig. 5c
  plotc<-levelplot(bioconcs_water, at=unique(c(seq(0, maxBioc, length=100))),
                   main="Fig 5c",
                   scales=list(fontfamily="Arial",x=list(rot=90)),
                   col.regions=colorFun(100), aspect=1,  contour=FALSE, pretty=TRUE,
                   border='black',
                   xlab="PCBs (ng/g WW)", 
                   ylab="Remediated Flow")
  
  #FBCs in scenario 2 levelplot, Fig. 5d
  plotd<-levelplot(fbcs_water_log, scales=list(fontfamily="Arial",x=list(rot=90)),
                   main="Fig 5d",
                   colorkey=list(at=key_points, labels=key_labels),
                   at=unique(c(seq(0, maxFBC, length=100))),
                   col.regions=topo.colors(130)[30:130], aspect=1,  contour=FALSE, pretty=TRUE,
                   xlab=expression("FBC" (mg/km^{2}/y)), border='black',
                   ylab="Remediated Flow")
  
  #plot link densities in scenario 2
  plot(lds_water,type="l", lwd=1, xlab="", xaxt="n", 
       yaxt="n",ylab="",ylim=c(ld_min,ld_max))
  polygon(cbind(c(1,1:51,51),c(0,lds_water,0)),lwd=1, col="#104EFF")
  return(list(plota=plota,plotb=plotb,plotc=plotc,plotd=plotd))
}